@@ -72,6 +72,16 @@ VALID_KEYS = [
     'become', 'become_user', 'become_method',
 ]
 
+BLOCK_NAME_TO_ACTION_TYPE_MAP = {
+    'tasks': 'task',
+    'handlers': 'handler',
+    'pre_tasks': 'task',
+    'post_tasks': 'task',
+    'block': 'meta',
+    'rescue': 'meta',
+    'always': 'meta',
+}
+
 
 def load_plugins(directory):
     result = []
@@ -174,13 +184,15 @@ def play_children(basedir, item, parent_type, playbook_dir):
         'handlers': _taskshandlers_children,
     }
     (k, v) = item
+    play_library = os.path.join(os.path.abspath(basedir), 'library')
+    _load_library_if_exists(play_library)
+
     if k in delegate_map:
         if v:
-            v = template(
-                    os.path.abspath(basedir),
-                    v,
-                    dict(playbook_dir=os.path.abspath(basedir)),
-                    fail_on_undefined=False)
+            v = template(os.path.abspath(basedir),
+                         v,
+                         dict(playbook_dir=os.path.abspath(basedir)),
+                         fail_on_undefined=False)
             return delegate_map[k](basedir, k, v, parent_type)
     return []
 
@@ -215,6 +227,11 @@ def _roles_children(basedir, k, v, parent_type):
     return results
 
 
+def _load_library_if_exists(path):
+    if os.path.exists(path):
+        module_loader.add_directory(path)
+
+
 def _rolepath(basedir, role):
     role_path = None
 
@@ -243,9 +260,7 @@ def _rolepath(basedir, role):
             break
 
     if role_path:
-        role_modules = os.path.join(role_path, 'library')
-        if os.path.exists(role_modules):
-            module_loader.add_directory(role_modules)
+        _load_library_if_exists(os.path.join(role_path, 'library'))
 
     return role_path
 
@@ -375,11 +390,15 @@ def normalize_task_v1(task):
 
 
 def normalize_task(task, filename):
+    ansible_action_type = task.get('__ansible_action_type__', 'task')
+    if '__ansible_action_type__' in task:
+        del(task['__ansible_action_type__'])
     if ANSIBLE_VERSION < 2:
         task = normalize_task_v1(task)
     else:
         task = normalize_task_v2(task)
     task[FILENAME_KEY] = filename
+    task['__ansible_action_type__'] = ansible_action_type
     return task
 
 
@@ -400,7 +419,7 @@ def extract_from_list(blocks, candidates):
         for candidate in candidates:
             if candidate in block:
                 if isinstance(block[candidate], list):
-                    results.extend(block[candidate])
+                    results.extend(add_action_type(block[candidate], candidate))
                 elif block[candidate] is not None:
                     raise RuntimeError(
                         "Key '%s' defined, but bad value: '%s'" %
@@ -408,10 +427,18 @@ def extract_from_list(blocks, candidates):
     return results
 
 
+def add_action_type(actions, action_type):
+    results = list()
+    for action in actions:
+        action['__ansible_action_type__'] = BLOCK_NAME_TO_ACTION_TYPE_MAP[action_type]
+        results.append(action)
+    return results
+
+
 def get_action_tasks(yaml, file):
     tasks = list()
     if file['type'] in ['tasks', 'handlers']:
-        tasks = yaml
+        tasks = add_action_type(yaml, file['type'])
     else:
         tasks.extend(extract_from_list(yaml, ['tasks', 'handlers', 'pre_tasks', 'post_tasks']))
 
